@@ -236,14 +236,14 @@ calibrate_carbon         <- function(inname,
   if (write_to_file) {
     cal_df$timeBgn <- convert_POSIXct_to_NEONhdf5_time(cal_df$timeBgn)
     cal_df$timeEnd <- convert_POSIXct_to_NEONhdf5_time(cal_df$timeEnd)
-    setup_output_file(inname, outname, site, "co2")
-    write_carbon_calibration_data(outname, site, cal_df, method = method)
-    write_carbon_ambient_data(outname, site, ciso_subset_cal)
+    fid <- setup_output_file(inname, outname, site, "co2",
+                              attrs = ciso$attrs, keep_open = TRUE)
+    write_carbon_calibration_data(outname, site, cal_df, method = method,
+                                  fid = fid)
+    write_carbon_ambient_data(outname, site, ciso_subset_cal, fid = fid)
+    h5_close(fid)
 
     validate_output_file(inname, outname, site, "co2")
-
-    # one last invocation of hdf5 close all, for good luck
-    rhdf5::h5closeAll()
   } else { #export output directly
     out_data <- list()
     #convert time to NEON HDF5 time
