@@ -66,7 +66,6 @@
 #' @import dplyr
 #' @import neonUtilities
 #' @importFrom data.table rleidv
-#' @importFrom utils packageVersion
 calibrate_water       <- function(inname,
                                   outname,
                                   site,
@@ -121,14 +120,14 @@ calibrate_water       <- function(inname,
   if (write_to_file) {
     cal_df$timeBgn <- convert_POSIXct_to_NEONhdf5_time(cal_df$timeBgn)
     cal_df$timeEnd <- convert_POSIXct_to_NEONhdf5_time(cal_df$timeEnd)
-    setup_output_file(inname, outname, site, analyte = "h2o")
-    write_water_calibration_data(outname, site, cal_df)
-    write_water_ambient_data(outname, site, wiso_subset_cal)
+    fid <- setup_output_file(inname, outname, site, analyte = "h2o",
+                              attrs = wiso$attrs, keep_open = TRUE)
+    write_water_calibration_data(outname, site, cal_df, fid = fid)
+    write_water_ambient_data(outname, site, wiso_subset_cal, fid = fid)
+    h5_close(fid)
 
     validate_output_file(inname, outname, site, "h2o")
 
-    rhdf5::h5closeAll()
-
   } else {
     out_data <- list()
     #convert time to NEON HDF5 time
