@@ -22,7 +22,6 @@
 #' @export
 #'
 #' @importFrom stats setNames
-#' @importFrom utils packageVersion
 #' @importFrom magrittr %>%
 ingest_data <- function(inname,
                         analyte,
@@ -39,45 +38,31 @@ ingest_data <- function(inname,
   analyte <- validate_analyte(analyte)
   backupMethod <- FALSE
 
-  # read attributes from (first file in) inname
-  site <- rhdf5::h5ls(inname[1], recursive = 1)[1, 2]
-  attrs <- rhdf5::h5readAttributes(inname[1], name = paste0("/", site))
+  # read site name and attributes from (first file in) inname
+  # combined into sequential calls to avoid redundant file opens
+  site <- h5_ls(inname[1])$name[1]
+  attrs <- h5_read_attrs(inname[1], site)
+  # attrs are also returned in the output list to avoid re-reading later
 
   nheights <- attrs$LvlMeasTow
 
   if (analyte == "Co2") {
 
-    if (packageVersion("neonUtilities") >= "2.3.0") {
-      data <- try(neonUtilities::stackEddy(inname,
-                                       avg = amb_avg,
-                                       level = "dp01",
-                                       var = "isoCo2",
-                                       useFasttime = TRUE)[[1]], silent = TRUE)
-      if ("try-error" %in% class(data)) {
-        data <- neonUtilities::stackEddy(inname,
-                                             avg = 9,
-                                             level = "dp01",
-                                             var = "isoCo2",
-                                             useFasttime = TRUE)[[1]]
-        backupMethod <- TRUE
-
-      }
-    } else if (packageVersion("neonUtilities") >= "2.1.1" && # nocov start
-                 packageVersion("neonUtilities") < "2.3.0") {
-      data <- try(neonUtilities::stackEddy(inname,
-                                       avg = amb_avg,
-                                       level = "dp01",
-                                       var = "isoCo2")[[1]], silent = TRUE)
-      if ("try-error" %in% class(data)) {
-        data <- neonUtilities::stackEddy(inname,
-                                         avg = 9,
-                                         level = "dp01",
-                                         var = "isoCo2")[[1]]
-        backupMethod <- TRUE
-      }
-    } else {
-      stop("NEONiso >= 0.7.0 requires neonUtilities >= 2.1.1")
-    } # nocov end
+    data <- try(neonUtilities::stackEddy(inname,
+                                     avg = amb_avg,
+                                     level = "dp01",
+                                     var = "isoCo2",
+                                     useFasttime = TRUE,
+                                     runLocal = TRUE)[[1]], silent = TRUE)
+    if ("try-error" %in% class(data)) {
+      data <- neonUtilities::stackEddy(inname,
+                                           avg = 9,
+                                           level = "dp01",
+                                           var = "isoCo2",
+                                           useFasttime = TRUE,
+                                           runLocal = TRUE)[[1]]
+      backupMethod <- TRUE
+    }
 
     # filter data and remove rows that are all NaNs:
     data <- data %>%
@@ -127,30 +112,18 @@ ingest_data <- function(inname,
   } else if (analyte == "H2o") {
 
     # stack data available for a given site into a single timeseries.
-    if (packageVersion("neonUtilities") >= "2.3.0") {
-      data9 <- neonUtilities::stackEddy(inname,
-                                        level = "dp01",
-                                        var = "isoH2o",
-                                        avg = amb_avg,
-                                        useFasttime = TRUE)[[1]]
-      data3 <- neonUtilities::stackEddy(inname,
-                                        level = "dp01",
-                                        var = "isoH2o",
-                                        avg = ref_avg,
-                                        useFasttime = TRUE)[[1]]
-    } else if (packageVersion("neonUtilities") >= "2.1.1" && # nocov start
-                 packageVersion("neonUtilities") < "2.3.0") {
-      data9 <- neonUtilities::stackEddy(inname,
-                                        level = "dp01",
-                                        var = "isoH2o",
-                                        avg = amb_avg)[[1]]
-      data3 <- neonUtilities::stackEddy(inname,
-                                        level = "dp01",
-                                        var = "isoH2o",
-                                        avg = ref_avg)[[1]]
-    } else {
-      stop("NEONiso >= 0.7.0 requires neonUtilities >= 2.1.1")
-    } # nocov end
+    data9 <- neonUtilities::stackEddy(inname,
+                                      level = "dp01",
+                                      var = "isoH2o",
+                                      avg = amb_avg,
+                                      useFasttime = TRUE,
+                                      runLocal = TRUE)[[1]]
+    data3 <- neonUtilities::stackEddy(inname,
+                                      level = "dp01",
+                                      var = "isoH2o",
+                                      avg = ref_avg,
+                                      useFasttime = TRUE,
+                                      runLocal = TRUE)[[1]]
 
     # filter data and remove rows that are all NaNs:
     data9 <- data9 %>%
@@ -304,8 +277,8 @@ ingest_data <- function(inname,
     names(refe_out) <- paste0(names(refe_out), "_", avg_char)
   }
 
-  output <- list(ambi_out, refe_out, reference)
-  names(output) <- c("ambient", "reference", "refe_stacked")
+  output <- list(ambi_out, refe_out, reference, attrs)
+  names(output) <- c("ambient", "reference", "refe_stacked", "attrs")
 
   return(output)
 }
