@@ -18,22 +18,24 @@ fout2 <- tempfile()
 fout3 <- tempfile()
 
 test_that("calibrate_carbon returns no error", {
-
-  skip_on_cran()
-  # these tests could probably be made more useful!!
   
+  # test file only has 9-minute data, so pass avg = 9 to exercise
+
+  # the primary code path rather than silently falling back
   expect_no_error(calibrate_carbon(fin, fout1, "ONAQ",
+                                  avg = 9,
                                   method = "gainoffset",
                                   calibration_half_width = 0.5,
                                   force_cal_to_beginning = TRUE,
                                   force_cal_to_end = TRUE,
                                   gap_fill_parameters = FALSE,
                                   filter_ambient = TRUE,
-                                r2_thres = 0.95,
+                                  r2_thres = 0.95,
                                   correct_ref_data = TRUE,
                                   write_to_file = TRUE))
 
   expect_no_error(calibrate_carbon(fin, fout2, "ONAQ",
+                                   avg = 9,
                                    method = "linreg",
                                    calibration_half_width = 0.5,
                                    force_cal_to_beginning = TRUE,
@@ -44,8 +46,9 @@ test_that("calibrate_carbon returns no error", {
                                    correct_ref_data = TRUE,
                                    write_to_file = TRUE))
 
-  # these tests could probably be made more useful!!
+
   expect_no_error(calibrate_carbon(fin, "/dev/null", "ONAQ",
+                                   avg = 9,
                                    method = "gainoffset",
                                    calibration_half_width = 0.5,
                                    force_cal_to_beginning = TRUE,
@@ -55,8 +58,9 @@ test_that("calibrate_carbon returns no error", {
                                    r2_thres = 0.95,
                                    correct_ref_data = TRUE,
                                    write_to_file = FALSE))
-  
+
   expect_warning(calibrate_carbon(fin, "/dev/null", "ONAQ",
+                                  avg = 9,
                                   method = "Bowling_2003",
                                   calibration_half_width = 0.5,
                                   force_cal_to_beginning = TRUE,
@@ -72,9 +76,6 @@ test_that("calibrate_carbon returns no error", {
 fout3 <- tempfile()
 
 test_that("calibrate_water returns no error", {
-
-  skip_on_cran()
-  # these tests could probably be made more useful!!
   expect_no_error(calibrate_water(fin, "/dev/null", "ONAQ",
                                   correct_ref_data = TRUE,
                                   write_to_file = FALSE))
